/***********************************************************************************
Class SC88_Display

  This class object is responsible for painting an approximate representation of
a Roland Sound Canvas 88 LCD (plasma display). It also does some MIDI System Exclusive
(SYSEX) generation for sending MIDI messages to an actual SC88.

      You may freely distribute, modify, and use this source
    code for non-commercial purposes, so long as this notice
    is left intact.  Commercial use of this source code or 
    any part of it requires express written permission from
    Michael Leach <mike@pcez.com>

 **********************************************************************************/

#ifndef __SC88_HPP
#define __SC88_HPP

#include <windows.h>
#include <stdio.h>

#define  SC_PART                1
#define  SC_INSTRUMENT          2
#define  SC_LEVEL               3
#define  SC_PAN                 4
#define  SC_REVERB              5
#define  SC_CHORUS              6
#define  SC_KEY_SHIFT           7
#define  SC_MIDI_CHANNEL        8
//GenerateMsg actions
#define  SC_SEND                0
#define  SC_SAVE                1

#define  SC_ROWS                16
#define  SC_COLUMNS             16

#define  BEGIN_SYSEX            0xF0
#define  END_SYSEX              0xF7
#define  DATA_OFFSET            5
#define  DISPLAY_MSG_SIZE       74
#define  MAX_MSG                2048

#define  SC_TOP                 20
#define  SC_LEFT                50
#define  DISPLAY_X              350
#define  DISPLAY_Y              140
#define  Y_INC                  30

//Menu IDs
#define  IDC_ALL_OFF_BUTTON         0
#define  IDC_ALL_ON_BUTTON          1
#define  IDC_DISPLAY_BUTTON         2
#define  IDC_STRING_BUTTON          3
#define  IDC_COMBOBOX               4
#define  IDC_EDIT_BOX               5
#define  IDC_SAVE_BUTTON            6

struct GS_PARAM{
   int       part;
   int       instr;
   char      patchName[16];
   int       level;
   int       pan;
   int       reverb;
   int       chorus;
   int       keyShift;
   char      MIDIBank;
   int       MIDIChan;
};

class SC88_Display{
private:
   HWND      hwnd;
   HBRUSH    bkBrush,
             gridOffBrush,
             gridOnBrush;
   RECT      rect,
             grid,
             dot[256];
   MIDIHDR   hdr;
   HMIDIOUT  hmo;
   GS_PARAM  gs;
   UINT      device;
   int       dotOn[256];
   unsigned char msg[DISPLAY_MSG_SIZE];

public:
   SC88_Display();
   SC88_Display(HWND hwindw) {hwnd = hwindw; };
   ~SC88_Display();
   void  SetWindow(HWND hwindw) { hwnd = hwindw; };
   void  DrawDisplay(void);
   void  UpdateGrid(POINT pt);
   RECT* GetGridRect(void) {return &grid; };
   void  AllOn(void);
   void  AllOff(void);
   void  GenerateDisplayMsg(int action);
   void  GenerateStringMsg(char *str, int action);
//   int SetValue(int type, int value); These functions are for future use. You can use
//   int GetValue(int type);            them as the read and write mechanism for musical MIDI data.
   void  SetDevice(UINT dev);
   UINT  GetDevice(void);
   unsigned char* GetDisplayBuffer(void);
   void  OpenMIDIOut(void);
   void  CloseMIDIOut(void);
   void  SendMessage(unsigned char* message, UINT size);
//   void  Save(void);
   int   SYSEXCheckSum(unsigned char*message);
};

// No need to jump around when these are called. Compile them inline with the code.
inline void SC88_Display::SetDevice(UINT dev) { device = dev; };
inline UINT SC88_Display::GetDevice(void)     { return device; };
inline unsigned char* SC88_Display::GetDisplayBuffer(void) {return msg; };

#endif
