/*------------------------------------------------------------
  SPEN.C   Sound Pen for the Roland Sound Canvas

This is the main initialization and message handler for the SC88 Display.

        You may freely distribute, modify, and use this source
    code for non-commercial purposes, so long as this notice
    is left intact.  Commercial use of this source code or 
    any part of it requires express written permission from
    Michael Leach <mike@pcez.com>

  ------------------------------------------------------------*/

#include <windows.h>
#include "sc88.hpp"

LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;
void SetComboStrings(HWND hCombo);
SC88_Display  SC88;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "Sound Pen" ;
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASSEX  wndclass ;

     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (GRAY_BRUSH) ;
     wndclass.lpszMenuName  = NULL ;
     wndclass.lpszClassName = szAppName ;
     wndclass.hIconSm       = LoadIcon (hInstance, "spen") ;

     RegisterClassEx (&wndclass) ;

     hwnd = CreateWindow (szAppName,         // window class name
                            "Sound Pen",     // window caption
                    WS_OVERLAPPEDWINDOW,     // window style
                    CW_USEDEFAULT,           // initial x position
                    CW_USEDEFAULT,           // initial y position
                    750,                     // initial x size
                    250,                     // initial y size
                    NULL,                    // parent window handle
                    NULL,                    // window menu handle
                    hInstance,               // program instance handle
                            NULL) ;          // creation parameters

     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;

     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
 	  return msg.wParam ;
     }

LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
   {
   POINT  pt;
   UINT   midiDevice = 0;
   char   str[80];

   static HWND hwndOnButton,
               hwndOffButton,
               hwndDisplayButton,
               hwndStringButton,
               hwndSaveButton,
               hwndCombo,
               hwndEdit;

   switch (iMsg)
      {
      case WM_CREATE :    //Setup all the buttons and menus.
         SC88.SetWindow(hwnd);
         hwndOnButton = CreateWindow("button", "All On",
            WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_DEFPUSHBUTTON,
            SC_LEFT+DISPLAY_X + 20, 21, 60, 20, hwnd,
            (HMENU)IDC_ALL_ON_BUTTON, ((LPCREATESTRUCT)lParam)->hInstance,
            NULL);
         hwndOffButton = CreateWindow("button", "All Off",
            WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_DEFPUSHBUTTON,
            SC_LEFT+DISPLAY_X + 20, 42, 60, 20, hwnd,
            (HMENU)IDC_ALL_OFF_BUTTON, ((LPCREATESTRUCT)lParam)->hInstance,
            NULL);
         hwndDisplayButton = CreateWindow("button", "Apply",
            WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_DEFPUSHBUTTON,
            SC_LEFT+DISPLAY_X + 20, 63, 60, 20, hwnd,
            (HMENU)IDC_DISPLAY_BUTTON, ((LPCREATESTRUCT)lParam)->hInstance,
            NULL);
         hwndSaveButton = CreateWindow("button", "Save",
            WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_DEFPUSHBUTTON,
            SC_LEFT+DISPLAY_X + 100, 63, 60, 20, hwnd,
            (HMENU)IDC_SAVE_BUTTON, ((LPCREATESTRUCT)lParam)->hInstance,
            NULL);
         hwndStringButton = CreateWindow("button", "Send",
            WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_DEFPUSHBUTTON,
            SC_LEFT + 260, SC_TOP + DISPLAY_Y + 30, 60, 25, hwnd,
            (HMENU)IDC_STRING_BUTTON, ((LPCREATESTRUCT)lParam)->hInstance, NULL);
         hwndCombo = CreateWindow ("combobox", NULL, WS_CHILD | WS_VISIBLE |
            WS_TABSTOP | CBS_DROPDOWN, SC_LEFT+DISPLAY_X + 85, 21, 250, 200,
            hwnd,(HMENU) IDC_COMBOBOX, ((LPCREATESTRUCT)lParam)->hInstance, 0);
         hwndEdit  = CreateWindow("edit", NULL, WS_CHILD | WS_VISIBLE |
            WS_BORDER | ES_LEFT | WS_TABSTOP, SC_LEFT, SC_TOP+ DISPLAY_Y + 30,
            250, 20, hwnd,
            (HMENU)IDC_EDIT_BOX, ((LPCREATESTRUCT)lParam)->hInstance, NULL);
         SetComboStrings(hwndCombo);
		   SendMessage(hwndCombo, CB_SETCURSEL, 0, 0);
         SC88.SetDevice(midiDevice);
         SC88.OpenMIDIOut();
         return 0 ;

      case WM_LBUTTONDOWN :
         pt.x = LOWORD (lParam);
         pt.y = HIWORD (lParam);
         SC88.UpdateGrid(pt);
         return 0;

      case WM_PAINT :
         SC88.DrawDisplay();
         return 0;

      case WM_MOVE :
         SC88.DrawDisplay();
         return 0;

      case WM_SIZE :
         SC88.DrawDisplay();
         return 0;

      case WM_COMMAND :
         if(HIWORD (wParam) == BN_CLICKED){
            switch(LOWORD (wParam)){
               case IDC_ALL_ON_BUTTON:
                  SC88.AllOn();
                  break;
               case IDC_ALL_OFF_BUTTON:
                  SC88.AllOff();
                  break;
               case IDC_DISPLAY_BUTTON:
                  midiDevice = SendMessage(hwndCombo, CB_GETCURSEL, 0, 0);
                  if(midiDevice != SC88.GetDevice()){
                     SC88.CloseMIDIOut();
                     SC88.SetDevice(midiDevice);
                     SC88.OpenMIDIOut();
                  }
                  SC88.GenerateDisplayMsg(SC_SEND);
                  break;
               case IDC_STRING_BUTTON:
                  GetWindowText(hwndEdit, str, 80);
                  SC88.GenerateStringMsg(str, SC_SEND);
                  break;
               case IDC_SAVE_BUTTON:
                  SC88.GenerateDisplayMsg(SC_SAVE);
                  GetWindowText(hwndEdit, str, 80);
                  SC88.GenerateStringMsg(str, SC_SAVE);
                  MessageBox(hwnd,
                  "Display dots saved as DOTS.SYX and\ndisplay string saved as TEXT.SYX.",
                  "Files Saved", MB_OK);
                  break;   
            }
         }
         break;

      case WM_DESTROY :
         PostQuitMessage (0) ;
         return 0 ;
      }

   return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
   }

void SetComboStrings(HWND hCombo){
   MIDIOUTCAPS  caps;
   UINT devs;
   UINT i;

   devs = midiOutGetNumDevs();
   for(i = 0; i < devs; i++){
      midiOutGetDevCaps(i, &caps, sizeof(MIDIOUTCAPS));
//      if(caps.wTechnology & MOD_MIDIPORT) Uncomment this if you want to get picky about
	                                     // what drivers SPEN will use....
         SendMessage(hCombo, CB_INSERTSTRING, i, (LPARAM) caps.szPname);
   }   
}

